--***************************************************************
--* File         : DUMPHEX.vhd                                  *
--* Created      : time-stamp: <08/04/03 01:01:32 kawasan>      *
--* Modified     : Time-stamp: <08/04/03 03:04:57 kawasan>      *
--*                                                             *
--* DUMPHEX                                                     *
--*                                                             *
--* Copyright 2008 Osamu Kawashima. all rights reserved	        *
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity DUMPHEX is
	generic (
		SADD_SIZE       : integer   := 32
	);
	port (
		RESET           : in  std_logic;
		SYSCLK          : in  std_logic;

		S_SADD          : in  std_logic_vector(SADD_SIZE-1 downto 0);
		S_EADD          : in  std_logic_vector(SADD_SIZE-1 downto 0);
		GO              : in  std_logic;
		DONE            : out std_logic;

		S_ADD           : out std_logic_vector(SADD_SIZE-1 downto 0);
		S_DI            : in  std_logic_vector(7 downto 0);
		S_GO            : out std_logic;
		S_DONE          : in  std_logic;

		D_DO            : out std_logic_vector(7 downto 0);
		D_GO            : out std_logic;
		D_DONE          : in  std_logic
	);
end DUMPHEX;

architecture RTL of DUMPHEX is

	constant ZERO32		: std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	type STATE is (S_STANDBY,S_NULL,
			S_READ,S_READ_DONE,
			S_OUT0,S_OUT1,S_OUT2,S_OUT3,
			S_LAST,
			S_OUT,S_OUT_DONE);
	signal next_state       : STATE;
	signal return_state     : STATE;

	signal s_go_is          : std_logic;
	signal s_add_is         : std_logic_vector(SADD_SIZE-1 downto 0);
	signal d_go_is          : std_logic;
	signal s_dat            : std_logic_vector(8 downto 0);

begin

	-- State-Machine --
	process (SYSCLK)
	begin
		if SYSCLK'event and SYSCLK='1' then
			if RESET='1' then
				next_state <= S_STANDBY;
				return_state <= S_NULL;
				DONE <= '1';
				s_add_is <= (others=>'0');
				s_go_is <= '0';
				d_go_is <= '0';
				s_dat <= (others=>'0');
			else
				case next_state is
					when S_STANDBY =>
						if GO='1' then
							next_state <= S_READ;
							DONE <= '0';
							s_add_is <= S_SADD;
						else
							DONE <= '1';
						end if;

					when S_READ =>
						next_state <= S_READ_DONE;
						s_go_is <= '1';

					when S_READ_DONE =>
						if S_DONE='1' then
							next_state <= S_OUT0;
						end if;
						s_go_is <= '0';

					when S_OUT0 =>
						next_state <= S_OUT;
						return_state <= S_OUT1;
						s_dat <= '1' & ZERO32(7 downto 4) & S_DI(7 downto 4);

					when S_OUT1 =>
						next_state <= S_OUT;
						return_state <= S_OUT2;
						s_dat <= '1' & ZERO32(7 downto 4) & S_DI(3 downto 0);

					when S_OUT2 =>
						next_state <= S_OUT;
						if s_add_is(3 downto 0)="1111" then
							s_dat <= '0' & x"0D";		-- '\r'
							return_state <= S_OUT3;
						else
							s_dat <= '0' & x"20";		-- ' '
							return_state <= S_LAST;
						end if;

					when S_OUT3 =>
						next_state <= S_OUT;
						return_state <= S_LAST;
						s_dat <= '0' & x"0A";		-- '\n'

					when S_LAST =>
						if s_add_is=S_EADD then
							next_state <= S_STANDBY;
						else
							next_state <= S_READ;
							s_add_is <= s_add_is + 1;
						end if;

					when S_OUT =>
						next_state <= S_OUT_DONE;
						d_go_is <= '1';

					when S_OUT_DONE =>
						if D_DONE='1' then
							next_state <= return_state;
						end if;
						d_go_is <= '0';

					when others  =>
						next_state <= S_STANDBY;
				end case;
			end if;
		end if;
	end process;
	-- State-Machine --

	process (s_dat)
	begin
		if s_dat(8)='0' then
			D_DO <= s_dat(7 downto 0);
		else
			case s_dat(3 downto 0) is
				when "0000" =>
					D_DO <= x"30";

				when "0001" =>
					D_DO <= x"31";

				when "0010" =>
					D_DO <= x"32";

				when "0011" =>
					D_DO <= x"33";

				when "0100" =>
					D_DO <= x"34";

				when "0101" =>
					D_DO <= x"35";

				when "0110" =>
					D_DO <= x"36";

				when "0111" =>
					D_DO <= x"37";

				when "1000" =>
					D_DO <= x"38";

				when "1001" =>
					D_DO <= x"39";

				when "1010" =>
					D_DO <= x"41";		-- 'A'

				when "1011" =>
					D_DO <= x"42";		-- 'B'

				when "1100" =>
					D_DO <= x"43";		-- 'C'

				when "1101" =>
					D_DO <= x"44";		-- 'D'

				when "1110" =>
					D_DO <= x"45";		-- 'E'

				when "1111" =>
					D_DO <= x"46";		-- 'F'

				when others  =>
					D_DO <= (others=>'X');
			end case;
		end if;
	end process;

	S_ADD <= s_add_is;
	S_GO <= s_go_is;
	D_GO <= d_go_is;

end RTL;
